@@ -1,4 +1,4 @@
-/* vi:set ts=8 sts=4 sw=4:
+/* vi:set ts=8 sts=4 sw=4 noet:
  *
  * VIM - Vi IMproved	by Bram Moolenaar
  *
@@ -66,6 +66,7 @@
 #define ADDR_BUFFERS		4
 #define ADDR_TABS		5
 #define ADDR_QUICKFIX		6
+#define ADDR_OTHER		99
 
 #ifndef DO_DECLARE_EXCMD
 typedef struct exarg exarg_T;
@@ -258,6 +259,9 @@ EX(CMD_catch,		"catch",	ex_catch,
 EX(CMD_cbuffer,		"cbuffer",	ex_cbuffer,
 			BANG|RANGE|NOTADR|WORD1|TRLBAR,
 			ADDR_LINES),
+EX(CMD_cbottom,		"cbottom",	ex_cbottom,
+			TRLBAR,
+			ADDR_LINES),
 EX(CMD_cc,		"cc",		ex_cc,
 			RANGE|NOTADR|COUNT|TRLBAR|BANG,
 			ADDR_LINES),
@@ -306,6 +310,9 @@ EX(CMD_checkpath,	"checkpath",	ex_checkpath,
 EX(CMD_checktime,	"checktime",	ex_checktime,
 			RANGE|NOTADR|BUFNAME|COUNT|EXTRA|TRLBAR,
 			ADDR_LINES),
+EX(CMD_chistory,	"chistory",	qf_history,
+			TRLBAR,
+			ADDR_LINES),
 EX(CMD_clist,		"clist",	qf_list,
 			BANG|EXTRA|TRLBAR|CMDWIN,
 			ADDR_LINES),
@@ -315,6 +322,9 @@ EX(CMD_clast,		"clast",	ex_cc,
 EX(CMD_close,		"close",	ex_close,
 			BANG|RANGE|NOTADR|COUNT|TRLBAR|CMDWIN,
 			ADDR_WINDOWS),
+EX(CMD_clearjumps,	"clearjumps",	ex_clearjumps,
+			TRLBAR|CMDWIN,
+			ADDR_LINES),
 EX(CMD_cmap,		"cmap",		ex_map,
 			EXTRA|TRLBAR|NOTRLCOM|USECTRLV|CMDWIN,
 			ADDR_LINES),
@@ -381,10 +391,10 @@ EX(CMD_cquit,		"cquit",	ex_cquit,
 EX(CMD_crewind,		"crewind",	ex_cc,
 			RANGE|NOTADR|COUNT|TRLBAR|BANG,
 			ADDR_LINES),
-EX(CMD_cscope,		"cscope",	do_cscope,
+EX(CMD_cscope,		"cscope",	ex_cscope,
 			EXTRA|NOTRLCOM|XFILE,
 			ADDR_LINES),
-EX(CMD_cstag,		"cstag",	do_cstag,
+EX(CMD_cstag,		"cstag",	ex_cstag,
 			BANG|TRLBAR|WORD1,
 			ADDR_LINES),
 EX(CMD_cunmap,		"cunmap",	ex_unmap,
@@ -534,6 +544,9 @@ EX(CMD_files,		"files",	buflist_list,
 EX(CMD_filetype,	"filetype",	ex_filetype,
 			EXTRA|TRLBAR|CMDWIN,
 			ADDR_LINES),
+EX(CMD_filter,		"filter",	ex_wrongmodifier,
+			BANG|NEEDARG|EXTRA|NOTRLCOM,
+			ADDR_LINES),
 EX(CMD_find,		"find",		ex_find,
 			RANGE|NOTADR|BANG|FILE1|EDITCMD|ARGOPT|TRLBAR,
 			ADDR_LINES),
@@ -610,7 +623,7 @@ EX(CMD_highlight,	"highlight",	ex_highlight,
 			BANG|EXTRA|TRLBAR|SBOXOK|CMDWIN,
 			ADDR_LINES),
 EX(CMD_hide,		"hide",		ex_hide,
-			BANG|RANGE|NOTADR|COUNT|EXTRA|NOTRLCOM,
+			BANG|RANGE|NOTADR|COUNT|EXTRA|TRLBAR,
 			ADDR_WINDOWS),
 EX(CMD_history,		"history",	ex_history,
 			EXTRA|TRLBAR|CMDWIN,
@@ -720,6 +733,9 @@ EX(CMD_later,		"later",	ex_later,
 EX(CMD_lbuffer,		"lbuffer",	ex_cbuffer,
 			BANG|RANGE|NOTADR|WORD1|TRLBAR,
 			ADDR_LINES),
+EX(CMD_lbottom,		"lbottom",	ex_cbottom,
+			TRLBAR,
+			ADDR_LINES),
 EX(CMD_lcd,		"lcd",		ex_cd,
 			BANG|FILE1|TRLBAR|CMDWIN,
 			ADDR_LINES),
@@ -729,7 +745,7 @@ EX(CMD_lchdir,		"lchdir",	ex_cd,
 EX(CMD_lclose,		"lclose",	ex_cclose,
 			RANGE|NOTADR|COUNT|TRLBAR,
 			ADDR_LINES),
-EX(CMD_lcscope,		"lcscope",	do_cscope,
+EX(CMD_lcscope,		"lcscope",	ex_cscope,
 			EXTRA|NOTRLCOM|XFILE,
 			ADDR_LINES),
 EX(CMD_ldo,		"ldo",		ex_listdo,
@@ -774,6 +790,9 @@ EX(CMD_lgrepadd,	"lgrepadd",	ex_make,
 EX(CMD_lhelpgrep,	"lhelpgrep",	ex_helpgrep,
 			EXTRA|NOTRLCOM|NEEDARG,
 			ADDR_LINES),
+EX(CMD_lhistory,	"lhistory",	qf_history,
+			TRLBAR,
+			ADDR_LINES),
 EX(CMD_ll,		"ll",		ex_cc,
 			RANGE|NOTADR|COUNT|TRLBAR|BANG,
 			ADDR_LINES),
@@ -886,8 +905,8 @@ EX(CMD_menutranslate,	"menutranslate", ex_menutranslate,
 			EXTRA|TRLBAR|NOTRLCOM|USECTRLV|CMDWIN,
 			ADDR_LINES),
 EX(CMD_messages,	"messages",	ex_messages,
-			TRLBAR|CMDWIN,
-			ADDR_LINES),
+			EXTRA|TRLBAR|RANGE|CMDWIN,
+			ADDR_OTHER),
 EX(CMD_mkexrc,		"mkexrc",	ex_mkrc,
 			BANG|FILE1|TRLBAR|CMDWIN,
 			ADDR_LINES),
@@ -1230,7 +1249,7 @@ EX(CMD_scriptnames,	"scriptnames",	ex_scriptnames,
 EX(CMD_scriptencoding,	"scriptencoding", ex_scriptencoding,
 			WORD1|TRLBAR|CMDWIN,
 			ADDR_LINES),
-EX(CMD_scscope,		"scscope",	do_scscope,
+EX(CMD_scscope,		"scscope",	ex_scscope,
 			EXTRA|NOTRLCOM,
 			ADDR_LINES),
 EX(CMD_set,		"set",		ex_set,
