@@ -1,108 +1,85 @@
-/* vi:set ts=8 sts=4 sw=4:
+/* vi:set ts=8 sts=4 sw=4 noet:
  *
  * VIM - Vi IMproved	by Bram Moolenaar
  *
  * Do ":help uganda"  in Vim to read copying and usage conditions.
  * Do ":help credits" in Vim to see a list of people who contributed.
+ * See README.txt for an overview of the Vim source code.
  */
 
 /*
  * farsi.c: functions for Farsi language
- *
- * Included by main.c, when FKMAP is defined.
  */
 
-static int toF_Xor_X_ __ARGS((int c));
-static int F_is_TyE __ARGS((int c));
-static int F_is_TyC_TyD __ARGS((int c));
-static int F_is_TyB_TyC_TyD __ARGS((int src, int offset));
-static int toF_TyB __ARGS((int c));
-static void put_curr_and_l_to_X __ARGS((int c));
-static void put_and_redo __ARGS((int c));
-static void chg_c_toX_orX __ARGS((void));
-static void chg_c_to_X_orX_ __ARGS((void));
-static void chg_c_to_X_or_X __ARGS((void));
-static void chg_l_to_X_orX_ __ARGS((void));
-static void chg_l_toXor_X __ARGS((void));
-static void chg_r_to_Xor_X_ __ARGS((void));
-static int toF_leading __ARGS((int c));
-static int toF_Rjoin __ARGS((int c));
-static int canF_Ljoin __ARGS((int c));
-static int canF_Rjoin __ARGS((int c));
-static int F_isterm __ARGS((int c));
-static int toF_ending __ARGS((int c));
-static void lrswapbuf __ARGS((char_u *buf, int len));
+#include "vim.h"
+
+#if defined(FEAT_FKMAP) || defined(PROTO)
+
+static int toF_Xor_X_(int c);
+static int F_is_TyE(int c);
+static int F_is_TyC_TyD(int c);
+static int F_is_TyB_TyC_TyD(int src, int offset);
+static int toF_TyB(int c);
+static void put_curr_and_l_to_X(int c);
+static void put_and_redo(int c);
+static void chg_c_toX_orX(void);
+static void chg_c_to_X_orX_(void);
+static void chg_c_to_X_or_X(void);
+static void chg_l_to_X_orX_(void);
+static void chg_l_toXor_X(void);
+static void chg_r_to_Xor_X_(void);
+static int toF_leading(int c);
+static int toF_Rjoin(int c);
+static int canF_Ljoin(int c);
+static int canF_Rjoin(int c);
+static int F_isterm(int c);
+static int toF_ending(int c);
+static void lrswapbuf(char_u *buf, int len);
 
 /*
-** Convert the given Farsi character into a _X or _X_ type
+ * Convert the given Farsi character into a _X or _X_ type
 */
     static int
-toF_Xor_X_(c)
-    int	c;
+toF_Xor_X_(int c)
 {
     int tempc;
 
     switch (c)
     {
-	case BE:
-		return _BE;
-	case PE:
-		return _PE;
-	case TE:
-		return _TE;
-	case SE:
-		return _SE;
-	case JIM:
-		return _JIM;
-	case CHE:
-		return _CHE;
-	case HE_J:
-		return _HE_J;
-	case XE:
-		return _XE;
-	case SIN:
-		return _SIN;
-	case SHIN:
-		return _SHIN;
-	case SAD:
-		return _SAD;
-	case ZAD:
-		return _ZAD;
-	case AYN:
-		return _AYN;
-	case AYN_:
-		return _AYN_;
-	case GHAYN:
-		return _GHAYN;
-	case GHAYN_:
-		return _GHAYN_;
-	case FE:
-		return _FE;
-	case GHAF:
-		return _GHAF;
-	case KAF:
-		return _KAF;
-	case GAF:
-		return _GAF;
-	case LAM:
-		return _LAM;
-	case MIM:
-		return _MIM;
-	case NOON:
-		return _NOON;
+	case BE: return _BE;
+	case PE: return _PE;
+	case TE: return _TE;
+	case SE: return _SE;
+	case JIM: return _JIM;
+	case CHE: return _CHE;
+	case HE_J: return _HE_J;
+	case XE: return _XE;
+	case SIN: return _SIN;
+	case SHIN: return _SHIN;
+	case SAD: return _SAD;
+	case ZAD: return _ZAD;
+	case AYN: return _AYN;
+	case AYN_: return _AYN_;
+	case GHAYN: return _GHAYN;
+	case GHAYN_: return _GHAYN_;
+	case FE: return _FE;
+	case GHAF: return _GHAF;
+	case KAF: return _KAF;
+	case GAF: return _GAF;
+	case LAM: return _LAM;
+	case MIM: return _MIM;
+	case NOON: return _NOON;
 	case YE:
-	case YE_:
-		return _YE;
+	case YE_: return _YE;
 	case YEE:
-	case YEE_:
-		return _YEE;
+	case YEE_: return _YEE;
 	case IE:
-	case IE_:
-		return _IE;
+	case IE_: return _IE;
 	case F_HE:
 		tempc = _HE;
 
-		if (p_ri && (curwin->w_cursor.col+1 < STRLEN(ml_get_curline())))
+		if (p_ri && (curwin->w_cursor.col + 1
+					 < (colnr_T)STRLEN(ml_get_curline())))
 		{
 		    inc_cursor();
 
@@ -127,95 +104,65 @@ toF_Xor_X_(c)
 }
 
 /*
-** Convert the given Farsi character into Farsi capital character .
+ * Convert the given Farsi character into Farsi capital character.
 */
-    int
-toF_TyA(c)
-    int	c ;
+    static int
+toF_TyA(int c)
 {
     switch (c)
     {
-	case ALEF_:
-		return ALEF;
-	case ALEF_U_H_:
-		return ALEF_U_H;
-	case _BE:
-		return BE;
-	case _PE:
-		return PE;
-	case _TE:
-		return TE;
-	case _SE:
-		return SE;
-	case _JIM:
-		return JIM;
-	case _CHE:
-		return CHE;
-	case _HE_J:
-		return HE_J;
-	case _XE:
-		return XE;
-	case _SIN:
-		return SIN;
-	case _SHIN:
-		return SHIN;
-	case _SAD:
-		return SAD;
-	case _ZAD:
-		return ZAD;
+	case ALEF_: return ALEF;
+	case ALEF_U_H_: return ALEF_U_H;
+	case _BE: return BE;
+	case _PE: return PE;
+	case _TE: return TE;
+	case _SE: return SE;
+	case _JIM: return JIM;
+	case _CHE: return CHE;
+	case _HE_J: return HE_J;
+	case _XE: return XE;
+	case _SIN: return SIN;
+	case _SHIN: return SHIN;
+	case _SAD: return SAD;
+	case _ZAD: return ZAD;
 	case _AYN:
 	case AYN_:
-	case _AYN_:
-		return AYN;
+	case _AYN_: return AYN;
 	case _GHAYN:
 	case GHAYN_:
-	case _GHAYN_:
-		return GHAYN;
-	case _FE:
-		return FE;
-	case _GHAF:
-		return GHAF;
-/* I am not sure what it is !!!	    case _KAF_H: */
-	case _KAF:
-		return KAF;
-	case _GAF:
-		return GAF;
-	case _LAM:
-		return LAM;
-	case _MIM:
-		return MIM;
-	case _NOON:
-		return NOON;
+	case _GHAYN_: return GHAYN;
+	case _FE: return FE;
+	case _GHAF: return GHAF;
+	/* I am not sure what it is !!!	    case _KAF_H: */
+	case _KAF: return KAF;
+	case _GAF: return GAF;
+	case _LAM: return LAM;
+	case _MIM: return MIM;
+	case _NOON: return NOON;
 	case _YE:
-	case YE_:
-		return YE;
+	case YE_: return YE;
 	case _YEE:
-	case YEE_:
-		return YEE;
-	case TEE_:
-		return TEE;
+	case YEE_: return YEE;
+	case TEE_: return TEE;
 	case _IE:
-	case IE_:
-		return IE;
+	case IE_: return IE;
 	case _HE:
-	case _HE_:
-		return F_HE;
+	case _HE_: return F_HE;
     }
     return c;
 }
 
 /*
-** Is the character under the cursor+offset in the given buffer a join type.
-** That is a character that is combined with the others.
-** Note: the offset is used only for command line buffer.
+ * Is the character under the cursor+offset in the given buffer a join type.
+ * That is a character that is combined with the others.
+ * Note: the offset is used only for command line buffer.
 */
     static int
-F_is_TyB_TyC_TyD(src, offset)
-    int	    src,    offset;
+F_is_TyB_TyC_TyD(int src, int offset)
 {
-int	c;
+    int		c;
 
-    if ( src == SRC_EDT)
+    if (src == SRC_EDT)
 	c = gchar_cursor();
     else
 	c = cmd_gchar(AT_CURSOR+offset);
@@ -259,11 +206,10 @@ int	c;
 }
 
 /*
-** Is the Farsi character one of the terminating only type.
+ * Is the Farsi character one of the terminating only type.
 */
     static int
-F_is_TyE(c)
-    int	    c;
+F_is_TyE(int c)
 {
     switch (c)
     {
@@ -283,11 +229,10 @@ F_is_TyE(c)
 }
 
 /*
-** Is the Farsi character one of the none leading type.
+ * Is the Farsi character one of the none leading type.
 */
     static int
-F_is_TyC_TyD(c)
-    int	    c;
+F_is_TyC_TyD(int c)
 {
     switch (c)
     {
@@ -308,16 +253,15 @@ F_is_TyC_TyD(c)
 }
 
 /*
-** Convert a none leading Farsi char into a leading type.
+ * Convert a none leading Farsi char into a leading type.
 */
     static int
-toF_TyB(c)
-    int	    c;
+toF_TyB(int c)
 {
     switch (c)
     {
 	case ALEF_:	return ALEF;
-	case ALEF_U_H_:	    return ALEF_U_H;
+	case ALEF_U_H_:	return ALEF_U_H;
 	case _AYN_:	return _AYN;
 	case AYN_:	return AYN;	/* exception - there are many of them */
 	case _GHAYN_:	return _GHAYN;
@@ -332,18 +276,17 @@ toF_TyB(c)
 }
 
 /*
-** Overwrite the current redo and cursor characters + left adjust
+ * Overwrite the current redo and cursor characters + left adjust.
 */
     static void
-put_curr_and_l_to_X(c)
-    int		  c;
+put_curr_and_l_to_X(int c)
 {
     int	tempc;
 
     if (curwin->w_p_rl && p_ri)
 	return;
 
-    if ( (curwin->w_cursor.col < STRLEN(ml_get_curline())))
+    if ((curwin->w_cursor.col < (colnr_T)STRLEN(ml_get_curline())))
     {
 	if ((p_ri && curwin->w_cursor.col) || !p_ri)
 	{
@@ -370,8 +313,7 @@ put_curr_and_l_to_X(c)
 }
 
     static void
-put_and_redo(c)
-    int c;
+put_and_redo(int c)
 {
     pchar_cursor(c);
     AppendCharToRedobuff(K_BS);
@@ -379,10 +321,10 @@ put_and_redo(c)
 }
 
 /*
-** Change the char. under the cursor to a X_ or X type
+ * Change the char. under the cursor to a X_ or X type
 */
     static void
-chg_c_toX_orX()
+chg_c_toX_orX(void)
 {
     int	tempc, curc;
 
@@ -503,11 +445,11 @@ chg_c_toX_orX()
 }
 
 /*
-** Change the char. under the cursor to a _X_ or X_ type
+ * Change the char. under the cursor to a _X_ or X_ type
 */
 
     static void
-chg_c_to_X_orX_()
+chg_c_to_X_orX_(void)
 {
     int	tempc;
 
@@ -555,16 +497,16 @@ chg_c_to_X_orX_()
 }
 
 /*
-** Change the char. under the cursor to a _X_ or _X type
+ * Change the char. under the cursor to a _X_ or _X type
 */
     static void
-chg_c_to_X_or_X ()
+chg_c_to_X_or_X (void)
 {
     int	tempc;
 
     tempc = gchar_cursor();
 
-    if (curwin->w_cursor.col+1 < STRLEN(ml_get_curline()))
+    if (curwin->w_cursor.col + 1 < (colnr_T)STRLEN(ml_get_curline()))
     {
 	inc_cursor();
 
@@ -581,20 +523,20 @@ chg_c_to_X_or_X ()
 	dec_cursor();
     }
 
-    if ((tempc = toF_Xor_X_(tempc)))
+    if ((tempc = toF_Xor_X_(tempc)) != 0)
 	put_and_redo(tempc);
 }
 
 /*
-** Change the character left to the cursor to a _X_ or X_ type
+ * Change the character left to the cursor to a _X_ or X_ type
 */
     static void
-chg_l_to_X_orX_ ()
+chg_l_to_X_orX_(void)
 {
     int	tempc;
 
-    if (!curwin->w_cursor.col &&
-	(curwin->w_cursor.col+1 == STRLEN(ml_get_curline())))
+    if (curwin->w_cursor.col != 0 &&
+	(curwin->w_cursor.col + 1 == (colnr_T)STRLEN(ml_get_curline())))
 	return;
 
     if (!curwin->w_cursor.col && p_ri)
@@ -654,16 +596,16 @@ chg_l_to_X_orX_ ()
 }
 
 /*
-** Change the charcter left to the cursor to a X or _X type
+ * Change the character left to the cursor to a X or _X type
 */
 
     static void
-chg_l_toXor_X ()
+chg_l_toXor_X (void)
 {
     int	tempc;
 
-    if (!curwin->w_cursor.col &&
-	(curwin->w_cursor.col+1 == STRLEN(ml_get_curline())))
+    if (curwin->w_cursor.col != 0 &&
+	(curwin->w_cursor.col + 1 == (colnr_T)STRLEN(ml_get_curline())))
 	return;
 
     if (!curwin->w_cursor.col && p_ri)
@@ -723,13 +665,13 @@ chg_l_toXor_X ()
 }
 
 /*
-** Change the charcter right to the cursor to a _X or _X_ type
+ * Change the character right to the cursor to a _X or _X_ type
 */
 
     static void
-chg_r_to_Xor_X_()
+chg_r_to_Xor_X_(void)
 {
-int tempc, c;
+    int tempc, c;
 
     if (curwin->w_cursor.col)
     {
@@ -738,7 +680,7 @@ int tempc, c;
 
 	tempc = gchar_cursor();
 
-	if ((c = toF_Xor_X_(tempc)))
+	if ((c = toF_Xor_X_(tempc)) != 0)
 	    put_and_redo(c);
 
 	if (!p_ri)
@@ -748,17 +690,19 @@ int tempc, c;
 }
 
 /*
-** Map Farsi keyboard when in fkmap mode.
+ * Map Farsi keyboard when in fkmap mode.
 */
 
     int
-fkmap(c)
-    int c;
+fkmap(int c)
 {
     int		tempc;
     static int	revins;
 
-    if (vim_isdigit(c) || ((c == '.' || c == '+' || c == '-' ||
+    if (IS_SPECIAL(c))
+	return c;
+
+    if (VIM_ISDIGIT(c) || ((c == '.' || c == '+' || c == '-' ||
 	c == '^' || c == '%' || c == '#' || c == '=')  && revins))
     {
 	if (!revins)
@@ -768,8 +712,8 @@ fkmap(c)
 		if (!p_ri)
 		    dec_cursor();
 
-		    chg_c_toX_orX ();
-		    chg_l_toXor_X ();
+		chg_c_toX_orX ();
+		chg_l_toXor_X ();
 
 		if (!p_ri)
 		    inc_cursor();
@@ -777,7 +721,7 @@ fkmap(c)
 	}
 
 	arrow_used = TRUE;
-	stop_arrow();
+	(void)stop_arrow();
 
 	if (!curwin->w_p_rl && revins)
 	    inc_cursor();
@@ -790,10 +734,9 @@ fkmap(c)
 	if (revins)
 	{
 	    arrow_used = TRUE;
-	    stop_arrow();
+	    (void)stop_arrow();
 
-	    revins=0;
-	    tempc = gchar_cursor();
+	    revins = 0;
 	    if (curwin->w_p_rl)
 	    {
 		while ((F_isdigit(gchar_cursor())
@@ -804,7 +747,7 @@ fkmap(c)
 				|| gchar_cursor() == F_DIVIDE
 				|| gchar_cursor() == F_PERCENT
 				|| gchar_cursor() == F_EQUALS))
-			&& gchar_cursor()!= NUL)
+			&& gchar_cursor() != NUL)
 		    ++curwin->w_cursor.col;
 	    }
 	    else
@@ -818,7 +761,8 @@ fkmap(c)
 				|| gchar_cursor() == F_DIVIDE
 				|| gchar_cursor() == F_PERCENT
 				|| gchar_cursor() == F_EQUALS))
-			    && --curwin->w_cursor.col);
+			    && --curwin->w_cursor.col)
+			;
 
 		if (!F_isdigit(gchar_cursor()))
 		    ++curwin->w_cursor.col;
@@ -903,8 +847,8 @@ fkmap(c)
 	    if (p_ri && c == NL && curwin->w_cursor.col)
 	    {
 		/*
-		** If the char before the cursor is _X_ or X_ do not change
-		** the one under the cursor with X type.
+		 * If the char before the cursor is _X_ or X_ do not change
+		 * the one under the cursor with X type.
 		*/
 
 		dec_cursor();
@@ -918,8 +862,7 @@ fkmap(c)
 		inc_cursor();
 	    }
 
-	    if (!p_ri)
-	    if (!curwin->w_cursor.col)
+	    if (!p_ri && !curwin->w_cursor.col)
 	    {
 		switch (c)
 		{
@@ -1158,43 +1101,42 @@ fkmap(c)
 
 	case 'a':
 		tempc = _SHIN;
-	    break;
+		break;
 	case 'A':
 		tempc = WAW_H;
-	    break;
+		break;
 	case 'b':
 		tempc = ZAL;
-	    break;
+		break;
 	case 'c':
 		tempc = ZE;
-	    break;
+		break;
 	case 'C':
 		tempc = JE;
-	    break;
+		break;
 	case 'd':
 		tempc = _YE;
-	    break;
+		break;
 	case 'D':
 		tempc = _YEE;
-	    break;
+		break;
 	case 'e':
 		tempc = _SE;
-	    break;
+		break;
 	case 'f':
 		tempc = _BE;
-	    break;
+		break;
 	case 'g':
 		tempc = _LAM;
-	    break;
+		break;
 	case 'G':
 	    if (!curwin->w_cursor.col  &&  STRLEN(ml_get_curline()))
-		{
+	    {
 
 		if (gchar_cursor() == _LAM)
 		    chg_c_toX_orX ();
-		else
-		    if (p_ri)
-			chg_c_to_X_or_X ();
+		else if (p_ri)
+		    chg_c_to_X_or_X ();
 	    }
 
 	    if (!p_ri)
@@ -1208,16 +1150,15 @@ fkmap(c)
 	    {
 		chg_c_toX_orX ();
 		chg_l_toXor_X ();
-		    tempc = ALEF_U_H;
+		tempc = ALEF_U_H;
+	    }
+	    else if (F_is_TyB_TyC_TyD(SRC_EDT, AT_CURSOR))
+	    {
+		tempc = ALEF_U_H_;
+		chg_l_toXor_X ();
 	    }
 	    else
-		if (F_is_TyB_TyC_TyD(SRC_EDT, AT_CURSOR))
-		{
-			tempc = ALEF_U_H_;
-		    chg_l_toXor_X ();
-		}
-		else
-			tempc = ALEF_U_H;
+		tempc = ALEF_U_H;
 
 	    if (!p_ri)
 		inc_cursor();
@@ -1247,17 +1188,17 @@ fkmap(c)
 		if (!p_ri)
 		    dec_cursor();
 
-		    tempc = LA;
+		tempc = LA;
 	    }
 	    else
 	    {
 		if (F_is_TyB_TyC_TyD(SRC_EDT, AT_CURSOR))
 		{
-			tempc = ALEF_;
+		    tempc = ALEF_;
 		    chg_l_toXor_X ();
 		}
 		else
-			tempc = ALEF;
+		    tempc = ALEF;
 	    }
 
 	    if (!p_ri)
@@ -1265,8 +1206,8 @@ fkmap(c)
 
 	    return tempc;
 	case 'i':
-	    if (!curwin->w_cursor.col  &&  STRLEN(ml_get_curline()))
-		{
+	    if (!curwin->w_cursor.col && STRLEN(ml_get_curline()))
+	    {
 		if (!p_ri && !F_is_TyE(tempc))
 		    chg_c_to_X_orX_ ();
 		if (p_ri)
@@ -1281,9 +1222,9 @@ fkmap(c)
 		dec_cursor();
 
 	    if (F_is_TyB_TyC_TyD(SRC_EDT, AT_CURSOR))
-		    tempc = _HE_;
+		tempc = _HE_;
 	    else
-		    tempc = _HE;
+		tempc = _HE;
 
 	    if (!p_ri)
 		inc_cursor();
@@ -1293,7 +1234,7 @@ fkmap(c)
 	    break;
 	case 'J':
 	    if (!curwin->w_cursor.col  &&  STRLEN(ml_get_curline()))
-		{
+	    {
 		if (p_ri)
 		    chg_c_to_X_or_X ();
 
@@ -1308,11 +1249,11 @@ fkmap(c)
 
 	    if (F_is_TyB_TyC_TyD(SRC_EDT, AT_CURSOR))
 	    {
-		    tempc = TEE_;
+		tempc = TEE_;
 		chg_l_toXor_X ();
 	    }
 	    else
-			tempc = TEE;
+		tempc = TEE;
 
 	    if (!p_ri)
 		inc_cursor();
@@ -1389,7 +1330,7 @@ fkmap(c)
 	    break;
 	case 'y':
 	    if (!curwin->w_cursor.col  &&  STRLEN(ml_get_curline()))
-		{
+	    {
 		if (!p_ri && !F_is_TyE(tempc))
 		    chg_c_to_X_orX_ ();
 		if (p_ri)
@@ -1438,7 +1379,7 @@ fkmap(c)
     if ((F_isalpha(tempc) || F_isdigit(tempc)))
     {
 	if (!curwin->w_cursor.col  &&  STRLEN(ml_get_curline()))
-	    {
+	{
 	    if (!p_ri && !F_is_TyE(tempc))
 		chg_c_to_X_orX_ ();
 	    if (p_ri)
@@ -1465,11 +1406,10 @@ fkmap(c)
 }
 
 /*
-** Convert a none leading Farsi char into a leading type.
+ * Convert a none leading Farsi char into a leading type.
 */
     static int
-toF_leading(c)
-    int	    c;
+toF_leading(int c)
 {
     switch (c)
     {
@@ -1520,11 +1460,10 @@ toF_leading(c)
 }
 
 /*
-** Convert a given Farsi char into right joining type.
+ * Convert a given Farsi char into right joining type.
 */
     static int
-toF_Rjoin(c)
-    int	    c;
+toF_Rjoin(int c)
 {
     switch (c)
     {
@@ -1577,11 +1516,10 @@ toF_Rjoin(c)
 }
 
 /*
-** Can a given Farsi character join via its left edj.
+ * Can a given Farsi character join via its left edj.
 */
     static int
-canF_Ljoin(c)
-    int	c;
+canF_Ljoin(int c)
 {
     switch (c)
     {
@@ -1652,11 +1590,10 @@ canF_Ljoin(c)
 }
 
 /*
-** Can a given Farsi character join via its right edj.
+ * Can a given Farsi character join via its right edj.
 */
     static int
-canF_Rjoin(c)
-    int	    c;
+canF_Rjoin(int c)
 {
     switch (c)
     {
@@ -1681,11 +1618,10 @@ canF_Rjoin(c)
 }
 
 /*
-** is a given Farsi character a terminating type.
+ * is a given Farsi character a terminating type.
 */
     static int
-F_isterm(c)
-    int	    c;
+F_isterm(int c)
 {
     switch (c)
     {
@@ -1709,106 +1645,73 @@ F_isterm(c)
 }
 
 /*
-** Convert the given Farsi character into a ending type .
+ * Convert the given Farsi character into a ending type .
 */
     static int
-toF_ending(c)
-    int	    c;
+toF_ending(int c)
 {
-
     switch (c)
     {
-	case _BE:
-		return BE;
-	case _PE:
-		return PE;
-	case _TE:
-		return TE;
-	case _SE:
-		return SE;
-	case _JIM:
-		return JIM;
-	case _CHE:
-		return CHE;
-	case _HE_J:
-		return HE_J;
-	case _XE:
-		return XE;
-	case _SIN:
-		return SIN;
-	case _SHIN:
-		return SHIN;
-	case _SAD:
-		return SAD;
-	case _ZAD:
-		return ZAD;
-	case _AYN:
-		return AYN;
-	case _AYN_:
-		return AYN_;
-	case _GHAYN:
-		return GHAYN;
-	case _GHAYN_:
-		return GHAYN_;
-	case _FE:
-		return FE;
-	case _GHAF:
-		return GHAF;
+	case _BE: return BE;
+	case _PE: return PE;
+	case _TE: return TE;
+	case _SE: return SE;
+	case _JIM: return JIM;
+	case _CHE: return CHE;
+	case _HE_J: return HE_J;
+	case _XE: return XE;
+	case _SIN: return SIN;
+	case _SHIN: return SHIN;
+	case _SAD: return SAD;
+	case _ZAD: return ZAD;
+	case _AYN: return AYN;
+	case _AYN_: return AYN_;
+	case _GHAYN: return GHAYN;
+	case _GHAYN_: return GHAYN_;
+	case _FE: return FE;
+	case _GHAF: return GHAF;
 	case _KAF_H:
-	case _KAF:
-		return KAF;
-	case _GAF:
-		return GAF;
-	case _LAM:
-		return LAM;
-	case _MIM:
-		return MIM;
-	case _NOON:
-		return NOON;
-	case _YE:
-		return YE_;
-	case YE_:
-		return YE;
-	case _YEE:
-		return YEE_;
-	case YEE_:
-		return YEE;
-	case TEE:
-		return TEE_;
-	case _IE:
-		return IE_;
-	case IE_:
-		return IE;
+	case _KAF: return KAF;
+	case _GAF: return GAF;
+	case _LAM: return LAM;
+	case _MIM: return MIM;
+	case _NOON: return NOON;
+	case _YE: return YE_;
+	case YE_: return YE;
+	case _YEE: return YEE_;
+	case YEE_: return YEE;
+	case TEE: return TEE_;
+	case _IE: return IE_;
+	case IE_: return IE;
 	case _HE:
-	case _HE_:
-		return F_HE;
+	case _HE_: return F_HE;
     }
     return c;
 }
 
 /*
-** Convert the Farsi 3342 standard into Farsi VIM.
+ * Convert the Farsi 3342 standard into Farsi VIM.
 */
-    void
-conv_to_pvim()
+    static void
+conv_to_pvim(void)
 {
-char_u *ptr;
-int lnum, llen, i;
+    char_u	*ptr;
+    int		lnum, llen, i;
 
     for (lnum = 1; lnum <= curbuf->b_ml.ml_line_count; ++lnum)
     {
-	ptr = ml_get((linenr_t)lnum);
+	ptr = ml_get((linenr_T)lnum);
 
-	llen = STRLEN(ptr);
+	llen = (int)STRLEN(ptr);
 
-	for ( i = 0; i < llen-1; i++)
+	for (i = 0; i < llen-1; i++)
 	{
 	    if (canF_Ljoin(ptr[i]) && canF_Rjoin(ptr[i+1]))
 	    {
 		ptr[i] = toF_leading(ptr[i]);
 		++i;
 
-		while(canF_Rjoin(ptr[i]) && (i < llen))
+		while (canF_Rjoin(ptr[i]) && i < llen)
 		{
 		    ptr[i] = toF_Rjoin(ptr[i]);
 		    if (F_isterm(ptr[i]) || !F_isalpha(ptr[i]))
@@ -1824,45 +1727,42 @@ int lnum, llen, i;
     }
 
     /*
-    ** Following lines contains Farsi encoded character.
-    */
+     * Following lines contains Farsi encoded character.
+     */
 
-    do_cmdline((char_u *)"%s/\202\231/\232/g", NULL, NULL, DOCMD_NOWAIT | DOCMD_VERBOSE);
-    do_cmdline((char_u *)"%s/\201\231/\370\334/g", NULL, NULL, DOCMD_NOWAIT | DOCMD_VERBOSE);
+    do_cmdline_cmd((char_u *)"%s/\202\231/\232/ge");
+    do_cmdline_cmd((char_u *)"%s/\201\231/\370\334/ge");
 
-    update_screen(CLEAR);
+    /* Assume the screen has been messed up: clear it and redraw. */
+    redraw_later(CLEAR);
     MSG_ATTR(farsi_text_1, hl_attr(HLF_S));
 }
 
 /*
-** Convert the Farsi VIM into Farsi 3342 standad.
-*/
-    void
-conv_to_pstd()
+ * Convert the Farsi VIM into Farsi 3342 standard.
+ */
+    static void
+conv_to_pstd(void)
 {
-char_u *ptr;
-int lnum, llen, i;
+    char_u	*ptr;
+    int		lnum, llen, i;
 
     /*
-    ** Following line contains Farsi encoded character.
-    */
-
-    do_cmdline((char_u *)"%s/\232/\202\231/g", NULL, NULL, DOCMD_NOWAIT | DOCMD_VERBOSE);
+     * Following line contains Farsi encoded character.
+     */
+    do_cmdline_cmd((char_u *)"%s/\232/\202\231/ge");
 
     for (lnum = 1; lnum <= curbuf->b_ml.ml_line_count; ++lnum)
     {
-	ptr = ml_get((linenr_t)lnum);
+	ptr = ml_get((linenr_T)lnum);
+	llen = (int)STRLEN(ptr);
 
-	llen = STRLEN(ptr);
-
-	for ( i = 0; i < llen; i++)
-	{
+	for (i = 0; i < llen; i++)
 	    ptr[i] = toF_TyA(ptr[i]);
-
-	}
     }
 
-    update_screen(CLEAR);
+    /* Assume the screen has been messed up: clear it and redraw. */
+    redraw_later(CLEAR);
     MSG_ATTR(farsi_text_2, hl_attr(HLF_S));
 }
 
@@ -1870,9 +1770,7 @@ int lnum, llen, i;
  * left-right swap the characters in buf[len].
  */
     static void
-lrswapbuf(buf, len)
-    char_u	*buf;
-    int		len;
+lrswapbuf(char_u *buf, int len)
 {
     char_u	*s, *e;
     int		c;
@@ -1894,8 +1792,7 @@ lrswapbuf(buf, len)
  * swap all the characters in reverse direction
  */
     char_u *
-lrswap(ibuf)
-    char_u	*ibuf;
+lrswap(char_u *ibuf)
 {
     if (ibuf != NULL && *ibuf != NUL)
 	lrswapbuf(ibuf, (int)STRLEN(ibuf));
@@ -1906,16 +1803,14 @@ lrswap(ibuf)
  * swap all the Farsi characters in reverse direction
  */
     char_u *
-lrFswap(cmdbuf, len)
-    char_u	*cmdbuf;
-    int		len;
+lrFswap(char_u *cmdbuf, int len)
 {
     int		i, cnt;
 
     if (cmdbuf == NULL)
 	return cmdbuf;
 
-    if (len == 0 && (len = STRLEN(cmdbuf)) == 0)
+    if (len == 0 && (len = (int)STRLEN(cmdbuf)) == 0)
 	return cmdbuf;
 
     for (i = 0; i < len; i++)
@@ -1933,19 +1828,20 @@ lrFswap(cmdbuf, len)
 }
 
 /*
- * Reverse the characters in the seach path and substitude section accordingly
+ * Reverse the characters in the search path and substitute section
+ * accordingly.
+ * TODO: handle different separator characters.  Use skip_regexp().
  */
     char_u *
-lrF_sub(ibuf)
-    char_u	*ibuf;
+lrF_sub(char_u *ibuf)
 {
     char_u	*p, *ep;
     int		i, cnt;
 
     p = ibuf;
 
-    /* Find the boundry of the search path */
-    while ((p = vim_strchr(++p, '/')) && p[-1] == '\\')
+    /* Find the boundary of the search path */
+    while (((p = vim_strchr(p + 1, '/')) != NULL) && p[-1] == '\\')
 	;
 
     if (p == NULL)
@@ -1954,11 +1850,11 @@ lrF_sub(ibuf)
     /* Reverse the Farsi characters in the search path. */
     lrFswap(ibuf, (int)(p-ibuf));
 
-    /* Now find the boundry of the substitute section */
-    if ((ep = (char_u *)strrchr((char *)++p, '/')))
-	cnt = ep - p;
+    /* Now find the boundary of the substitute section */
+    if ((ep = (char_u *)strrchr((char *)++p, '/')) != NULL)
+	cnt = (int)(ep - p);
     else
-	cnt = STRLEN(p);
+	cnt = (int)STRLEN(p);
 
     /* Reverse the characters in the substitute section and take care of '\' */
     for (i = 0; i < cnt-1; i++)
@@ -1977,10 +1873,9 @@ lrF_sub(ibuf)
  * Map Farsi keyboard when in cmd_fkmap mode.
  */
     int
-cmdl_fkmap(c)
-    int c;
+cmdl_fkmap(int c)
 {
-int	tempc;
+    int	    tempc;
 
     switch (c)
     {
@@ -2161,7 +2056,7 @@ int	tempc;
 	case 'f':   return _BE;
 	case 'g':   return _LAM;
 	case 'G':
-		    if (cmd_gchar(AT_CURSOR) == _LAM )
+		if (cmd_gchar(AT_CURSOR) == _LAM)
 		{
 		    cmd_pchar(LAM, AT_CURSOR);
 			    return ALEF_U_H;
@@ -2172,7 +2067,7 @@ int	tempc;
 		else
 			return ALEF_U_H;
 	case 'h':
-		    if (cmd_gchar(AT_CURSOR) == _LAM )
+		if (cmd_gchar(AT_CURSOR) == _LAM)
 		{
 		    cmd_pchar(LA, AT_CURSOR);
 		    redrawcmdline();
@@ -2228,77 +2123,73 @@ int	tempc;
 	case ',':   return WAW;
 	case '[':   return _JIM;
 	case ']':   return _CHE;
-	}
+    }
 
-	return c;
+    return c;
 }
 
 /*
  * F_isalpha returns TRUE if 'c' is a Farsi alphabet
  */
     int
-F_isalpha(c)
-    int	c;
+F_isalpha(int c)
 {
-    return (( c >= TEE_ && c <= _YE) ||
-	( c >= ALEF_A && c <= YE) ||
-	( c >= _IE && c <= YE_));
+    return ((c >= TEE_ && c <= _YE)
+	    || (c >= ALEF_A && c <= YE)
+	    || (c >= _IE && c <= YE_));
 }
 
 /*
  * F_isdigit returns TRUE if 'c' is a Farsi digit
  */
     int
-F_isdigit(c)
-    int	    c;
+F_isdigit(int c)
 {
-    return (( c >= FARSI_0 && c <= FARSI_9 ));
+    return (c >= FARSI_0 && c <= FARSI_9);
 }
 
 /*
  * F_ischar returns TRUE if 'c' is a Farsi character.
  */
     int
-F_ischar(c)
-    int	    c;
+F_ischar(int c)
 {
-    return (( c >= TEE_ && c <= YE_));
+    return (c >= TEE_ && c <= YE_);
 }
 
     void
-farsi_fkey(c)
-    int	    c;
+farsi_f8(cmdarg_T *cap UNUSED)
 {
-    if (c == K_F8)
+    if (p_altkeymap)
     {
-	if (p_altkeymap)
+	if (curwin->w_farsi & W_R_L)
 	{
-	    if (curwin->w_p_pers & W_R_L)
-	    {
-		p_fkmap=0;
-		do_cmdline((char_u *)"set norl", NULL, NULL, DOCMD_NOWAIT);
-		MSG("");
-	    }
-	    else
-	    {
-		p_fkmap=1;
-		do_cmdline((char_u *)"set rl", NULL, NULL, DOCMD_NOWAIT);
-		MSG("");
-	    }
-
-	    curwin->w_p_pers = curwin->w_p_pers ^ W_R_L;
+	    p_fkmap = 0;
+	    do_cmdline_cmd((char_u *)"set norl");
+	    MSG("");
 	}
-    }
-
-    if (c == K_F9)
-    {
-	if (p_altkeymap && curwin->w_p_rl)
+	else
 	{
-	    curwin->w_p_pers = curwin->w_p_pers ^ W_CONV;
-	    if (curwin->w_p_pers & W_CONV)
-		conv_to_pvim();
-	    else
-		conv_to_pstd();
+	    p_fkmap = 1;
+	    do_cmdline_cmd((char_u *)"set rl");
+	    MSG("");
 	}
+
+	curwin->w_farsi = curwin->w_farsi ^ W_R_L;
     }
 }
+
+    void
+farsi_f9(cmdarg_T *cap UNUSED)
+{
+    if (p_altkeymap && curwin->w_p_rl)
+    {
+	curwin->w_farsi = curwin->w_farsi ^ W_CONV;
+	if (curwin->w_farsi & W_CONV)
+	    conv_to_pvim();
+	else
+	    conv_to_pstd();
+    }
+}
+
+#endif /* FEAT_FKMAP */
