@@ -1,4 +1,4 @@
-/* vi:set ts=8 sts=4 sw=4:
+/* vi:set ts=8 sts=4 sw=4 noet:
  *
  * VIM - Vi IMproved	by Bram Moolenaar
  *
@@ -17,8 +17,8 @@
 # define signal sigset
 #endif
 
-   /* sun's sys/ioctl.h redefines symbols from termio world */
-#if defined(HAVE_SYS_IOCTL_H) && !defined(sun)
+   /* Sun's sys/ioctl.h redefines symbols from termio world */
+#if defined(HAVE_SYS_IOCTL_H) && !defined(SUN_SYSTEM)
 # include <sys/ioctl.h>
 #endif
 
@@ -28,11 +28,6 @@
 #  include <sys/wait.h>
 # endif
 
-#if defined(HAVE_SYS_SELECT_H) && \
-	(!defined(HAVE_SYS_TIME_H) || defined(SYS_SELECT_WITH_SYS_TIME))
-#  include <sys/select.h>
-# endif
-
 # ifndef WEXITSTATUS
 #  ifdef HAVE_UNION_WAIT
 #   define WEXITSTATUS(stat_val) ((stat_val).w_T.w_Retcode)
@@ -52,34 +47,23 @@
 #endif /* !USE_SYSTEM */
 
 #ifdef HAVE_STROPTS_H
+#ifdef sinix
+#define buf_T __system_buf_t__
+#endif
 # include <stropts.h>
+#ifdef sinix
+#undef buf_T
+#endif
 #endif
 
 #ifdef HAVE_STRING_H
 # include <string.h>
 #endif
 
-#ifndef HAVE_SELECT
-# ifdef HAVE_SYS_POLL_H
-#  include <sys/poll.h>
-# else
-#  ifdef HAVE_POLL_H
-#   include <poll.h>
-#  endif
-# endif
-#endif
-
 #ifdef HAVE_SYS_STREAM_H
 # include <sys/stream.h>
 #endif
 
-#ifdef HAVE_SYS_PTEM_H
-# include <sys/ptem.h>
-# ifndef _IO_PTEM_H	    /* For UnixWare that should check for _IO_PT_PTEM_H */
-#  define _IO_PTEM_H
-# endif
-#endif
-
 #ifdef HAVE_SYS_UTSNAME_H
 # include <sys/utsname.h>
 #endif
@@ -87,12 +71,18 @@
 #ifdef HAVE_SYS_SYSTEMINFO_H
 /*
  * foolish Sinix <sys/systeminfo.h> uses SYS_NMLN but doesn't include
- * limits.h>, where it is defined. Perhaps other systems have the same
+ * <limits.h>, where it is defined. Perhaps other systems have the same
  * problem? Include it here. -- Slootman
  */
 # if defined(HAVE_LIMITS_H) && !defined(_LIMITS_H)
 #  include <limits.h>		/* for SYS_NMLN (Sinix 5.41 / Unix SysV.4) */
 # endif
+
+/* Define SYS_NMLN ourselves if it still isn't defined (for CrayT3E). */
+# ifndef SYS_NMLN
+#  define SYS_NMLN 32
+# endif
+
 # include <sys/systeminfo.h>	/* for sysinfo */
 #endif
 
@@ -113,3 +103,23 @@
 #  endif
 # endif
 #endif
+
+#ifdef HAVE_SYS_PTEM_H
+# include <sys/ptem.h>	/* must be after termios.h for Sinix */
+# ifndef _IO_PTEM_H	/* For UnixWare that should check for _IO_PT_PTEM_H */
+#  define _IO_PTEM_H
+# endif
+#endif
+
+/* shared library access */
+#if defined(HAVE_DLFCN_H) && defined(USE_DLOPEN)
+# if defined(__MVS__) && !defined (__SUSV3)
+    /* needed to define RTLD_LAZY (Anthony Giorgio) */
+#  define __SUSV3
+# endif
+# include <dlfcn.h>
+#else
+# if defined(HAVE_DL_H) && defined(HAVE_SHL_LOAD)
+#  include <dl.h>
+# endif
+#endif
