@@ -6,6 +6,7 @@
 
 from __future__ import print_function
 import sys
+import time
 
 if __name__ == "__main__":
 
@@ -31,6 +32,15 @@ if __name__ == "__main__":
         if typed.startswith("double "):
             print(typed[7:-1] + "\nAND " + typed[7:-1])
             sys.stdout.flush()
+        if typed.startswith("split "):
+            print(typed[6:-1], end='')
+            sys.stdout.flush()
+            time.sleep(0.05)
+            print(typed[6:-1], end='')
+            sys.stdout.flush()
+            time.sleep(0.05)
+            print(typed[6:-1])
+            sys.stdout.flush()
         if typed.startswith("echoerr "):
             print(typed[8:-1], file=sys.stderr)
             sys.stderr.flush()
